################################################################
# ILOT DE CHALEUR DE SURFACE, à partir des LST MODIS (Land Surface Temperature)
# Ici utilisation des LST 8-day, pour les étés 2013 et 2014
################################################################

# Librairies nécessaires :
library(maptools)
library(rgdal)
library(raster)
library(rasterVis)
library(RColorBrewer)


# modification du chemin du répertoire de travail
setwd("~//Documents/ElementR_Raster/LST_MODIS")

# Récupération du listing des fichiers tiff
id <- dir(pattern = ".tif")
id
date <- substr(id,16,23)  # pour extraire la date du nom des fichiers
date

################################################################
# Manipulation rapide des rasters et carto
################################################################

# avec la librairie rgdal => format sp

# Par ex, pour la 1ère date
tmp <- readGDAL(id[1])
image(tmp[1])     # avec une visualisation de contrôle

# Quel est le systeme de coordonnees ici ?
str(tmp)   # ou lecture dans la fenetre Environment
# Combien de pixels ?

# Possibilite d'ajouter des bandes (i.e. d'autres rasters) ou des shp
# mais un p'tit pb : le poids de ce format pour les rasters
# Pour ex, avec 24 rasters, 740 Ko en geoTiff, 510 Ko en SpatialGridDataFrame 
# et seulement 2 Ko avec le format suivant

################################################################
# Avec la librairie raster

tmp <- raster(id[1])
# Le format est RasterLayer (avec info sur le système de coord, etc.)
image(tmp)
# la fonction image() permet de visualiser rapidement un raster (par ex, pour vérification, etc.)
# Mais si modification de la fenêtre, alors ajustement et donc déformation de "l'espace géographique"
spplot(tmp)

# Extraction de données du raster, comme avec une matrice
tmp[1,1]
tmp[,5]     # 5ème colonne de la matrice
tmp[2,]     # 2ème ligne de la matrice
tmp[2:4,c(1,3)] 
mean(tmp[,])

# Que voyez-vous sur la 21ème image ?
# quelle répartition spatiale des températures de surface ?
# à partir de la fonction spplot (de raster)
tmp <- raster(id[21]) 
spplot(tmp)

tmp[tmp==0 ] <- NA       # pour transformer tous les 0 en NA
 
pal <- rev(brewer.pal(11,"RdBu"))    # définition d'une palette qui va être réutilisée ensuite
spplot(tmp, cuts=11,col.regions=pal)

# Une autre manière à partir de rasterVis (librairie pour visualiser des rasters) 
levelplot(tmp,  par.settings=BuRdTheme)       # Si pb,    rasterVis::levelplot(tmp,  par.settings=BuRdTheme)

# Pour constituer un "RasterLayer" avec plusieurs bandes (ici plusieurs LST 8-day)
tmp2 <- stack(id)    # permet de faire une "pile" avec l'ensemble des images
nlayers(tmp2)
dim(tmp2)


# On peut directement traiter l'ensemble des bandes (i.e. des valeurs des rasters)
# 1/ Modifier les 0 en NA
tmp2[tmp2==0, ] <- NA

# vous pouvez aussi décider de retirer du lot des rasters avec trop de NA
tmp2 <- dropLayer(tmp2, 21)
nlayers(tmp2)

# 2/ Transformer les valeurs en °C
# Sur la documentation du produit MODIS utilisé MYD11A2 : 
# UNITS = Kelvin et MULTIPLY BY SCALE FACTOR = 0.02
tmp2 <- (tmp2 * 0.02)-273.15 

# 3/ "Cartographier" l'ensemble des img retenues
spplot(tmp2, cuts=11,col.regions=pal)

# Remarque : en climato, souvent on travaille non pas sur les valeurs "brutes", 
# mais sur les anomalies (non fait ici)

################################################################
# Relation entre LST moy et l'altitude ?
# ou comment manipuler des rasters avec des définitions différentes...
################################################################

# Calcul de la moyenne 
moyLST <- mean(tmp2, na.rm = T)
histogram(moyLST)    # en général, les T° ont plutôt une distribution normale / ici légère dissymétrie
spplot(moyLST, cuts=10, col.regions=pal)

# Import du fichier brut SRTM = Digital Elevation Model (pour chq pixel, une altitude)
# pour + d'infos : http://www2.jpl.nasa.gov/srtm/  et pour télécharger http://srtm.csi.cgiar.org/
srtm <- raster("../SRTM/srtm_37_03.tif")
spplot(srtm, col.regions=terrain.colors(20))   # avec la palette classique pour le relief
dim(srtm)

# Extraction de l'étendue de MODIS et ré-échantillonnage à la même résolution
srtm <- crop(srtm, extent(moyLST))
spplot(srtm, col.regions=terrain.colors(20))
dim(srtm)
srtm <- resample(srtm, moyLST, method='bilinear')
dim(srtm)

# Y a-t-il une relation entre la température de surface et l'altitude ?
# Si oui, vous pouvez enlever l'effet de l'altitude
plot(srtm, moyLST)
srtm_v <- as.vector(srtm[[1]])        # création de 2 vecteurs pour "individualiser" les valeurs des rasters 
moyLST_v <- as.vector(moyLST[[1]])
cor(srtm_v, moyLST_v)
res <- lm(srtm_v~moyLST_v)
res

################################################################
# Relation entre LST moy et l'occupation du sol ?
# ou comment combiner vecteur-raster, avec des projections différentes...
################################################################

# Import du fichier shp de CLC  (projeté en Lambert 93) et du fond communal
# Simplifié : 
# - 111 (Tissu urbain continu),
# - 112 (Tissu urbain discontinu), 
# - 1.. (le reste de Territoires artificialisés),
# - 2 (Territoires agricoles), 
# - 3 (Forêts et milieux semi-naturels), 
# - 4 (Zones humides), 
# - 5 (Surfaces en eau)

# Définition d'une projection Lambert 93
L93 <- CRS("+init=epsg:2154")   # CRS = Coordinate Ref System  // http://www.epsg-registry.org/

CLC <- readShapePoly("../CLC/CLC_Idf_simplif.shp", proj4string = L93)
FdComm <- readShapePoly("../Comm/FdComm_L93.shp", proj4string = L93)
plot(CLC)         # pour vérifier
plot(FdComm)      

# Projection du raster 
moyLST.L93 <- projectRaster(moyLST, crs = L93)
spplot(moyLST.L93) 

graphique <- spplot(moyLST.L93, cuts=11, col.regions=pal) 
graphique + layer(sp.lines(FdComm, col="black", lwd=0.5))

# Stats zonales
zones <- rasterize(CLC, field="CodeSimp", y=moyLST.L93 ,progress="text")   # avec un p'tit gadget pour voir l'évolution
spplot(zones, cuts=7, col.regions=brewer.pal(7,"Set1"))

boxplot(moyLST.L93, zones)

occsol <- c("1..", "111", "112", "2", "3", "4", "5")
mean <- zonal(moyLST.L93, zones, mean)[,2]
min <- zonal(moyLST.L93, zones, min)[,2]
max <- zonal(moyLST.L93, zones, max)[,2]
sd <- zonal(moyLST.L93, zones, sd)[,2]
StatsZonales <- cbind(occsol, mean, min, max, sd)
StatsZonales






